==> spirit_hub-0.0.1.10.tar.gz: spirit_hub-0.0.1.10/app/models/spirit_hub/operation.rb <==
# frozen_string_literal: true

module SpiritHub
  class Operation < ApplicationRecord
    include ::AclProtected

    field :code, type: String
    field :title, type: String
    field :city, type: String
    field :address, type: String
    field :postal_code, type: String
    field :agency, type: String
    field :type, type: String
    field :start_date, type: Date
    field :end_date, type: Date
    field :status, type: String

    belongs_to :user, foreign_key: :responsible
    belongs_to :user, foreign_key: :current_user_id
    belongs_to :tagset, foreign_key: :root_tagset_id, optional: true
    belongs_to :doc, foreign_key: :classement_plan_id, optional: true

    index({ code: 1 }, { unique: true, name: 'unique_code_index' })

    validates_presence_of :title, :city, :address, :postal_code,
                          :agency, :type, :start_date, :end_date
    validates :code, presence: true, uniqueness: { case_sensitive: false }
    validate :end_date_after_start_date

    after_create :generate_operation_tree
    before_save :format_code, :update_status
    after_update :updated_tagset_operation_title, :update_directories
    after_destroy :destroy_tagsets

    # TODO : Change when we have the exact need
    def update_status
      self.status = if start_date > Date.today
                      'A venir'
                    elsif end_date < Date.today
                      'Terminé'
                    else
                      'En cours'
                    end
    end

    def format_code
      self.code = code.upcase
    end

    def root_tagset
      Tagset.find_by(id: root_tagset_id)
    end

    def classement_plan
      Doc.find_by(id: classement_plan_id)
    end

    def phases
      Phase.where(operation_id: id)
    end

    # TODO : Move to core
    def writers?(user)
      (writers.to_a & user.all_gids_and_user).any?
    end

    def readers?(user)
      (readers.to_a & user.all_gids_and_user).any?
    end

    def responsible?(user)
      responsible&.to_s == user.id.to_s
    end

    private

    def generate_operation_tree
      return if !root_tagset_id.blank? || classement_plan_id.blank?

      SpiritHub::OperationTreeModel.generate_operation_tree(self)
    end

    def end_date_after_start_date
      return unless end_date.present? && start_date.present? && end_date < start_date

      errors.add(:end_date, 'doit être postérieure à la date de début')
    end

    def updated_tagset_operation_title
      return unless saved_change_to_attribute?(:title)

      root_tagset&.set(updated_at: DateTime.current, title: "Projet #{title}", operation_title: title)

      Tagset.where(operation_id: id).each do |tagset|
        tagset.set(updated_at: DateTime.current, operation_title: title)
      end
    end

    def update_directories
      updated_attributes = %i[responsible current_user_id start_date end_date].each_with_object({}) do |attr, changes|
        changes[attr] = send(attr) if saved_change_to_attribute?(attr)
      end

      return if updated_attributes.empty?

      Tagset.where(operation_id: id, type: 'directory').each do |tagset|
        tagset.update(updated_attributes)
      end
    end

    def destroy_tagsets
      Tagset.where(operation_id: id).each do |tagset|
        tagset.current_user_id = current_user_id
        tagset.destroy
      end
    end
  end
end

==> spirit_hub-0.0.1.13.tar.gz: spirit_hub-0.0.1.13/app/models/spirit_hub/operation.rb <==
# frozen_string_literal: true

module SpiritHub
  class Operation < ApplicationRecord
    include ::AclProtected

    field :code, type: String
    field :title, type: String
    field :city, type: String
    field :address, type: String
    field :postal_code, type: String
    field :agency, type: String
    field :type, type: String
    field :start_date, type: Date
    field :end_date, type: Date
    field :status, type: String

    belongs_to :user, foreign_key: :responsible
    belongs_to :user, foreign_key: :current_user_id
    belongs_to :tagset, foreign_key: :root_tagset_id, optional: true
    belongs_to :doc, foreign_key: :classement_plan_id, optional: true

    index({ code: 1 }, { unique: true, name: 'unique_code_index' })

    validates_presence_of :title, :city, :address, :postal_code,
                          :agency, :type, :start_date, :end_date
    validates :code, presence: true, uniqueness: { case_sensitive: false }
    validate :end_date_after_start_date

    before_save :format_code, :update_status
    after_update :updated_tagset_operation_title, :update_directories
    after_destroy :destroy_tagsets

    # TODO : Change when we have the exact need
    def update_status
      self.status = if start_date > Date.today
                      'A venir'
                    elsif end_date < Date.today
                      'Terminé'
                    else
                      'En cours'
                    end
    end

    def format_code
      self.code = code.upcase
    end

    def root_tagset
      Tagset.find_by(id: root_tagset_id)
    end

    def classement_plan
      Doc.find_by(id: classement_plan_id)
    end

    def phases
      Phase.where(operation_id: id)
    end

    def generate_operation_tree(user)
      return if !root_tagset_id.blank? || classement_plan_id.blank?

      SpiritHub::OperationTreeModel.generate_operation_tree(self, user)
    end

    # TODO : Move to core
    def writers?(user)
      (writers.to_a & user.all_gids_and_user).any?
    end

    def readers?(user)
      (readers.to_a & user.all_gids_and_user).any?
    end

    def responsible?(user)
      responsible&.to_s == user.id.to_s
    end

    private

    def end_date_after_start_date
      return unless end_date.present? && start_date.present? && end_date < start_date

      errors.add(:end_date, 'doit être postérieure à la date de début')
    end

    def updated_tagset_operation_title
      return unless saved_change_to_attribute?(:title)

      root_tagset&.set(updated_at: DateTime.current, title: "Projet #{title}", operation_title: title)

      Tagset.where(operation_id: id).each do |tagset|
        tagset.set(updated_at: DateTime.current, operation_title: title)
      end
    end

    def update_directories
      updated_attributes = %i[responsible current_user_id start_date end_date].each_with_object({}) do |attr, changes|
        changes[attr] = send(attr) if saved_change_to_attribute?(attr)
      end

      return if updated_attributes.empty?

      Tagset.where(operation_id: id, type: 'directory').each do |tagset|
        tagset.update(updated_attributes)
      end
    end

    def destroy_tagsets
      Tagset.where(operation_id: id).each do |tagset|
        tagset.current_user_id = current_user_id
        tagset.destroy
      end
    end
  end
end
